package cn.wise.sc.consume.business.controller;



import cn.wise.sc.consume.business.entity.SysCoupon;
import cn.wise.sc.consume.business.entity.SysCouponDetail;
import cn.wise.sc.consume.business.entity.SysUser;
import cn.wise.sc.consume.business.mapper.SysCouponDetailMapper;
import cn.wise.sc.consume.business.model.CouponDetailDTO;
import cn.wise.sc.consume.business.model.CouponDetailVO;
import cn.wise.sc.consume.business.model.CouponListVO;
import cn.wise.sc.consume.business.model.PageTableRequest;
import cn.wise.sc.consume.business.service.ISysCouponDetailService;
import cn.wise.sc.consume.business.service.ISysCouponService;
import cn.wise.sc.consume.business.service.impl.CommonService;
import cn.wise.sc.consume.business.util.ExcelUtil;
import cn.wise.sc.consume.business.util.UserUtil;
import cn.wise.sc.consume.business.wrapper.WrapMapper;
import cn.wise.sc.consume.business.wrapper.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;


import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author ztw
 * @since 2020-07-31
 */
@Api(tags = "优惠券管理")
@RestController
@RequestMapping("/business/sys-coupon")
public class SysCouponController {

	private static final Logger log = LoggerFactory.getLogger("SysCouponController");

	@Autowired
	private ISysCouponService couponService;
	@Autowired
	private ISysCouponDetailService couponDetailService;
	@Autowired
	private SysCouponDetailMapper couponDetailMapper;
	@Autowired
	private CommonService  commonService;

	@ApiOperation(value = "优惠券列表")
	@ApiImplicitParams(value = {
			@ApiImplicitParam(name = "current", value = "当前页数(从1开始)", paramType = "query", dataType = "Long"),
			@ApiImplicitParam(name = "size", value = "当前每页显示数", paramType = "query", dataType = "Long"),
			@ApiImplicitParam(name = "couponName", value = "优惠券名称", paramType = "query", dataType = "String"),
			@ApiImplicitParam(name = "enterpriseId", value = "所属企业", paramType = "query", dataType = "Long"),
			@ApiImplicitParam(name = "status", value = "状态：0禁用, 1启用", paramType = "query", dataType = "Integer")
	})
	@GetMapping("getCouponPage")
	public Wrapper getCouponPage(PageTableRequest request, String couponName, Long enterpriseId, Integer status) {
		try {
			couponDetailService.beVerdue();
			IPage<CouponListVO> page = new Page<>(request.getCurrent(), request.getSize());
			Map<String, Object> map = new HashMap<>();
			map.put("couponName", couponName);
			map.put("enterpriseId", enterpriseId);
			map.put("status", status);
			List<CouponListVO> couponListVOS = couponService.getPage(map);
			page.setRecords(couponListVOS).setTotal(couponListVOS.size());
			return WrapMapper.wrap(200,"成功",page);
		} catch (Exception e) {
			log.debug("优惠券列表{}", e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "添加优惠券")
	@PostMapping("/saveCoupon")
	public Wrapper saveCoupon(@RequestBody SysCoupon coupon){
		try {
			coupon.setStatus(1);
			coupon.setIsDelete(1);
			coupon.setCreateTime(LocalDateTime.now());
			couponService.save(coupon);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e){
			log.debug("添加优惠券{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "修改优惠券")
	@PostMapping("/updateCoupon")
	public Wrapper updateCoupon(@RequestBody SysCoupon coupon){
		try {
			SysCoupon c = couponService.getById(coupon.getId());
			if(c == null){
				return WrapMapper.wrap(400,"信息错误");
			}
			couponService.updateById(coupon);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e){
			log.debug("修改优惠券{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "优惠券详情")
	@GetMapping("coupon/{id}")
	public Wrapper getCouponById(@PathVariable Long id){
		try {
			CouponDetailVO c = couponService.getCouponDetail(id);
			if(c == null){
				return WrapMapper.wrap(400,"信息错误");
			}
			return WrapMapper.wrap(200,"成功",c);
		}catch (Exception e){
			log.debug("优惠券详情{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "修改优惠券状态")
	@PostMapping("/updateStatus/{id}")
	public Wrapper updateStatus(@PathVariable Long id) {
		try {
			SysCoupon e = couponService.getById(id);
			if (e == null ) {
				return WrapMapper.wrap(400,"优惠券不存在");
			}
			Integer status = e.getStatus()==0?1:0;
			e.setStatus(status);
			couponService.updateById(e);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e) {
			log.debug("修改优惠券状态{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "删除优惠券")
	@PostMapping("/deleteCoupon/{id}")
	public Wrapper deleteCoupon(@PathVariable Long id) {
		try {
			SysCoupon c = couponService.getById(id);
			if (c == null ) {
				return WrapMapper.wrap(400,"优惠券不存在");
			}
			c.setIsDelete(0);
			c.setStatus(0);
			couponService.updateById(c);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e) {
			log.debug("删除优惠券{}",e);
		}
		return WrapMapper.error("失败");
	}



	@ApiOperation(value = "优惠券详情列表")
	@ApiImplicitParams(value = {
			@ApiImplicitParam(name = "current", value = "当前页数(从1开始)", paramType = "query", dataType = "Long"),
			@ApiImplicitParam(name = "size", value = "当前每页显示数", paramType = "query", dataType = "Long"),
			@ApiImplicitParam(name = "isReceive", value = "票券状态：0未领取，1已领取，2已过期", paramType = "query", dataType = "Integer"),
			@ApiImplicitParam(name = "couponId", value = "优惠券表id", paramType = "query", dataType = "Long")
	})
	@GetMapping("/getCouponDetailPage")
	public Wrapper getCouponDetailPage(PageTableRequest request, Integer isReceive, Long couponId) {
		try {
			if(couponId == null){
				return WrapMapper.wrap(400,"请选择优惠券信息");
			}
			QueryWrapper<SysCouponDetail> wrapper = new QueryWrapper<>();
			wrapper.eq("coupon_id", couponId);
			if (isReceive != null) {
				wrapper.eq("is_receive", isReceive);
			}
			wrapper.eq("is_delete", 1);
			wrapper.orderByDesc("cmb_no");
			IPage<SysCouponDetail> page = new Page<>(request.getCurrent(), request.getSize());
			page = couponDetailMapper.selectPage(page, wrapper);
			return WrapMapper.wrap(200, "成功", page);

		} catch (Exception e) {
			log.debug("优惠券详情列表{}", e);
		}
		return WrapMapper.error("失败");
	}


	@ApiOperation(value = "添加优惠券详情")
	@PostMapping("/saveCouponDetail")
	public Wrapper saveCouponDetail(@RequestBody CouponDetailDTO couponDetailDTO){
		try {
			int counts = couponDetailMapper.createNo();
			SysCouponDetail couponDetail = new SysCouponDetail();
			couponDetail.setCouponId(couponDetailDTO.getCouponId())
					.setCmbNo(commonService.createNo("cmb_",counts))
					.setCouponCode(couponDetailDTO.getCouponCode())
					.setStartDate(couponDetailDTO.getStartDate())
					.setEndDate(couponDetailDTO.getEndDate())
					.setIsReceive(0)
					.setCreateTime(LocalDateTime.now())
					.setIsDelete(1)
					.setRemark(couponDetailDTO.getRemark());
			couponDetailService.save(couponDetail);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e){
			log.debug("添加优惠券详情{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "修改优惠券详情")
	@PostMapping("/updateCouponDetail")
	public Wrapper updateCouponDetail(@RequestBody CouponDetailDTO couponDetailDTO){
		try {
			SysCouponDetail couponDetail = couponDetailService.getById(couponDetailDTO.getId());
			if(couponDetail == null){
				return WrapMapper.wrap(400,"信息错误");
			}
			couponDetail.setStartDate(couponDetailDTO.getStartDate())
					.setEndDate(couponDetailDTO.getEndDate())
					.setIsReceive(0)
					.setRemark(couponDetailDTO.getRemark());
			couponDetailService.updateById(couponDetail);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e){
			log.debug("修改优惠券详情{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "单张优惠券详情")
	@GetMapping("couponDetail/{id}")
	public Wrapper getcouponDetailById(@PathVariable Long id){
		try {
			SysCouponDetail couponDetail = couponDetailService.getById(id);
			if(couponDetail == null){
				return WrapMapper.wrap(400,"信息错误");
			}
			return WrapMapper.wrap(200,"成功",couponDetail);
		}catch (Exception e){
			log.debug("优惠券详情{}",e);
		}
		return WrapMapper.error("失败");
	}


	@ApiOperation(value = "删除单张优惠券详情")
	@PostMapping("/deleteCouponDetail/{id}")
	public Wrapper deleteCouponDetail(@PathVariable Long id) {
		try {
			SysCouponDetail couponDetail = couponDetailService.getById(id);
			if (couponDetail == null ) {
				return WrapMapper.wrap(400,"优惠券详情不存在");
			}
			couponDetail.setIsDelete(0);
			couponDetailService.updateById(couponDetail);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e) {
			log.debug("删除单张优惠券详情{}",e);
		}
		return WrapMapper.error("失败");
	}

	protected void download(Resource resource, HttpServletResponse response) {
		response.setHeader("content-type", "application/octet-stream");
		response.setContentType("application/vnd.ms-excel;charset=utf-8");
		try {
			response.setHeader("Content-Disposition", "attachment;filename="+ new String((resource.getFilename()).getBytes(), "iso-8859-1"));
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
		InputStream in = null;
		OutputStream out = null;
		try {
			in = resource.getInputStream();
			int len = 0;
			byte[] buffer = new byte[1024];
			out = response.getOutputStream();
			while ((len = in.read(buffer)) > 0) {
				out.write(buffer, 0, len);
			}

		} catch (Exception e) {
			throw new RuntimeException(e);
		} finally {
			if (in != null) {
				try {
					in.close();
				} catch (Exception e) {
					throw new RuntimeException(e);
				}
			}
			try {
				out.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	@ApiOperation(value = "下载优惠券模版", httpMethod = "GET", notes = "下载优惠券模版")
	@RequestMapping(value = "/download", method = RequestMethod.GET)
	public void downloadFile(HttpServletResponse response) throws IOException {
		Resource resource = new ClassPathResource("couponDetail.xlsx");
		download(resource, response);
	}


	@ApiOperation(value = "导入优惠券信息", httpMethod = "POST", notes = "导入优惠券信息")
	@RequestMapping(value = "/importCouponDetail", method = RequestMethod.POST)
	public Wrapper importCouponDetail( @RequestBody MultipartFile file,Long couponId) {
		// 当前登录用户
		SysUser sysUser = UserUtil.getLoginUser();
		if (sysUser == null) {// 用户不存在
			return WrapMapper.wrap(400,"用户不存在");
		}
		if(couponId == null){
			return WrapMapper.wrap(400,"请选择优惠券信息");
		}
		try {
			InputStream in = file.getInputStream();
			List<Map<Integer, String>> conponDetailmp = ExcelUtil.readData((FileInputStream) in);

			List<SysCouponDetail> conponDetailList = new ArrayList<>();

			for (Map<Integer, String> hm : conponDetailmp) {
				SysCouponDetail couponDetail = new SysCouponDetail();
				if ((hm.get(0) == null || "".equals(hm.get(0))) &&
						(hm.get(1) == null || "".equals(hm.get(1))) &&
						(hm.get(2) == null || "".equals(hm.get(2))) &&
						(hm.get(3) == null || "".equals(hm.get(3))) &&
						(hm.get(4) == null || "".equals(hm.get(4)))
						) {
					continue;
				}
				if (hm.get(0) == null || "".equals(hm.get(0)) ||
						hm.get(1) == null || "".equals(hm.get(1)) ||
						hm.get(2) == null || "".equals(hm.get(2)) ||
						hm.get(3) == null || "".equals(hm.get(3))
						) {
					return WrapMapper.wrap(400,"数据信息错误");
				}
				couponDetail.setCouponCode(hm.get(1))
						.setStartDate(LocalDate.parse(hm.get(2), DateTimeFormatter.ofPattern("yyyy/MM/dd")))
						.setEndDate(LocalDate.parse(hm.get(3), DateTimeFormatter.ofPattern("yyyy/MM/dd")))
						.setRemark(hm.get(4));
				conponDetailList.add(couponDetail);
			}
			for (SysCouponDetail couponDetail : conponDetailList) {
				int counts = couponDetailMapper.createNo();
				couponDetail.setCouponId(couponId)
						.setCmbNo(commonService.createNo("cmb_",counts))
						.setIsReceive(0)
						.setCreateTime(LocalDateTime.now())
						.setIsDelete(1);
				couponDetailService.save(couponDetail);
			}
			return WrapMapper.wrap(200,"成功");
		} catch (Exception e) {
			e.printStackTrace();
			log.debug("导入发票信息{}",e);
		}
		return WrapMapper.error("失败");
	}










}

