package cn.wise.sc.consume.business.controller;


import cn.wise.sc.consume.business.entity.SysEnterprise;
import cn.wise.sc.consume.business.mapper.SysEnterpriseMapper;
import cn.wise.sc.consume.business.model.PageTableRequest;
import cn.wise.sc.consume.business.service.ISysEnterpriseService;
import cn.wise.sc.consume.business.util.RedisUtil;
import cn.wise.sc.consume.business.wrapper.WrapMapper;
import cn.wise.sc.consume.business.wrapper.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author ztw
 * @since 2020-06-17
 */
@Api(tags = "企业管理")
@RestController
@RequestMapping("/business/sys-enterprise")
public class SysEnterpriseController {
	private static final Logger log = LoggerFactory.getLogger("SysEnterpriseController");

	@Autowired
	RedisUtil redisUtil;
	@Autowired
	private ISysEnterpriseService enterpriseService;
	@Resource
	private SysEnterpriseMapper enterpriseMapper;


	public String redisExample() {
		redisUtil.setString("token", new Object(), 2000);
		return "sucess";
	}

	@ApiOperation(value = "企业列表")
	@ApiImplicitParams(value = {
			@ApiImplicitParam(name = "current", value = "当前页数(从1开始)", paramType = "query", dataType = "Long"),
			@ApiImplicitParam(name = "size", value = "当前每页显示数", paramType = "query", dataType = "Long"),
			@ApiImplicitParam(name = "enterpriseName", value = "企业名称", paramType = "query", dataType = "String"),
			@ApiImplicitParam(name = "status", value = "状态：0禁用, 1启用，2待审批", paramType = "query", dataType = "Integer")
	})
	@GetMapping("/getList")
	public Wrapper getList(PageTableRequest request, String enterpriseName, Integer status) {
		try {
			QueryWrapper<SysEnterprise> wrapper = new QueryWrapper<>();
			if (StringUtils.isNotBlank(enterpriseName)) {
				wrapper.like("enterprise_name", enterpriseName);
			}
			if (status != null) {
				wrapper.eq("status", status);
			}
			wrapper.eq("is_delete", 1);
			wrapper.orderByDesc("create_time");
			IPage<SysEnterprise> page = new Page<>(request.getCurrent(), request.getSize());
			page = enterpriseMapper.selectPage(page, wrapper);
			return WrapMapper.wrap(200, "成功", page);

		} catch (Exception e) {
			log.debug("企业列表{}", e);
		}
		return WrapMapper.error("失败");

	}

	@ApiOperation(value = "添加企业")
	@PostMapping("/save")
	public Wrapper save(@RequestBody SysEnterprise enterprise){
		try {
			enterprise.setStatus(1);
			enterprise.setIsDelete(1);
			enterprise.setCreateTime(LocalDateTime.now());
			enterpriseService.save(enterprise);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e){
			log.debug("添加企业{}",e);
		}
		return WrapMapper.error("失败");
	}


	@ApiOperation(value = "修改企业")
	@PostMapping("/update")
	public Wrapper update(@RequestBody SysEnterprise enterprise){
		try {
			SysEnterprise e = enterpriseService.getById(enterprise.getId());
			if(e == null){
				return WrapMapper.wrap(400,"信息错误");
			}
			enterpriseService.updateById(enterprise);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e){
			log.debug("修改企业{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "企业详情")
	@GetMapping("{id}")
	public Wrapper getById(@PathVariable Long id){
		try {
			SysEnterprise e = enterpriseService.getById(id);
			if(e == null){
				return WrapMapper.wrap(400,"信息错误");
			}
			return WrapMapper.wrap(200,"成功",e);
		}catch (Exception e){
			log.debug("企业详情{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "修改企业状态")
	@PostMapping("/updateStatus/{id}")
	public Wrapper updateStatus(@PathVariable Long id) {
		try {
			SysEnterprise e = enterpriseService.getById(id);
			if (e == null ) {
				return WrapMapper.wrap(400,"企业不存在");
			}
			Integer status = e.getStatus()==0?1:0;
			e.setStatus(status);
			enterpriseService.updateById(e);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e) {
			log.debug("修改企业状态{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "删除企业")
	@PostMapping("/{id}")
	public Wrapper delete(@PathVariable Long id) {
		try {
			SysEnterprise e = enterpriseService.getById(id);
			if (e == null ) {
				return WrapMapper.wrap(400,"企业不存在");
			}
			e.setStatus(0);
			e.setIsDelete(0);
			enterpriseService.updateById(e);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e) {
			log.debug("删除企业{}",e);
		}
		return WrapMapper.error("失败");
	}

	@ApiOperation(value = "审批企业入驻申请")
	@PostMapping("/approval")
	public Wrapper approval(Long id) {
		try {
			SysEnterprise e = enterpriseService.getById(id);
			if (e == null ) {
				return WrapMapper.wrap(400,"企业不存在");
			}
			e.setStatus(1);
			enterpriseService.updateById(e);
			return WrapMapper.wrap(200,"成功");
		}catch (Exception e) {
			log.debug("审批企业入驻申请{}",e);
		}
		return WrapMapper.error("失败");
	}



	//****************************************************************************************************

	/*@ApiOperation(value = "企业列表（自定义sql实现）")
	@ApiImplicitParams(value = {
			@ApiImplicitParam(name = "current", value = "当前页数(从1开始)", paramType = "query", dataType = "Long"),
			@ApiImplicitParam(name = "size", value = "当前每页显示数", paramType = "query", dataType = "Long"),
			@ApiImplicitParam(name = "enterpriseName", value = "企业名称", paramType = "query", dataType = "String"),
			@ApiImplicitParam(name = "status", value = "状态：0禁用, 1启用", paramType = "query", dataType = "Integer")
	})
	@GetMapping("getList1")
	public Wrapper getList1(PageTableRequest request, String enterpriseName, Integer status){
		try {
			Page<SysEnterprise> page = new Page<>(request.getCurrent(), request.getSize());
			Map<String, Object> map = new HashMap<>();
			map.put("enterpriseName", enterpriseName);
			map.put("status", status);
			List<SysEnterprise> enterpriseList = enterpriseService.getPage(map);
			page.setRecords(enterpriseList).setTotal(enterpriseList.size());
			return WrapMapper.wrap(200,"成功",page);
		}catch (Exception e){
			log.debug("企业列表{}",e);
		}
		return WrapMapper.error("失败");

	}*/




}

